<?php // Include only
if (!defined('BOOT')) exit('No bootstrap.  Exiting.');

/*
Wax Poteto database layer Copyright 2011-2015 Marc "Waccoon" Leveille
http://www.NineChime.com/products/
Version 5.9.1 - Last modified 2015-08-21

Notes:
	As of 2015-08-21, PHP5.3 is now required, and some updates have removed code that checks for PHP < 5.2.  MySQL 5.0.8 is still the target minimum.

	PHP has deprecated MySQL in favor of MySQLi, but Wax still supports MySQL calls as many people using it depend on legacy systems and some mods still depend on MySQL calls.  For PHP7, the hack setting 'FORCE_MYSQL_API' is ignored, and loud errors will be reported for mods still relying on the old MySQL extension.

	OO style is not used for MySQLi, because the MySQL extension doesn't support it, and all DB calls in Wax are procedural style.
*/


// Init
$dbconn    = null;
$db_time_queries = 0.0;

// Config must be public for backwards compatibility
if (!defined('PARSE_DBCONN_MANUALLY') || !PARSE_DBCONN_MANUALLY) {
	require 'dbconn.php';
}

// Remap prefixes
$db_p   = $OekakiPoteto_Prefix;
$db_mem = $OekakiPoteto_MemberPrefix;
$db_op  = $db_p;   // Deprecated
$db_pre = $db_p;   // Deprecated

// Do not use outside of db_layer.php!
$db_error   = array();
$db_history = array();


// Use MySQL or MySQLi?  MySQL is only available on < PHP7
$si = true;

if (defined('FORCE_MYSQL_API') && FORCE_MYSQL_API == 1) {
	if (version_compare(PHP_VERSION, '7.0.0', '<')) {
		$si = false;
	}
}



function db_open($force = false) {
	global $si, $cfg_db, $db_p;
	global $dbhost, $dbuser, $dbpass, $db_error, $dbname;

	$db_type = 'MySQL';

	if ($si) {
		$db_type = 'MySQLi';

		$GLOBALS['dbconn'] = @mysqli_connect($dbhost, $dbuser, $dbpass);
		if (!$GLOBALS['dbconn']) {
			db_push_error('Could not connect to the SQL database (API: MySQLi)<br />DB: '.mysqli_connect_error() );
		}
	} else {
		$GLOBALS['dbconn'] = mysql_connect($dbhost, $dbuser, $dbpass);
		if (!$GLOBALS['dbconn']) {
			db_push_error('Could not connect to the SQL database (API: MySQL)<br />DB: '.mysql_error() );
		}
	}

	if (!$GLOBALS['dbconn']) {
		if ($force === false) {
			exit(db_error() );
		} else {
			return false;
		}
	}

	if (!db_select($GLOBALS['dbconn'], $dbname)) {
		db_close();
		db_push_error('Database name &ldquo;'.htmlspecialchars($dbname).'&rdquo; is invalid (API: '.$db_type.')');

		if ($force === false) {
			exit(db_error() );
		} else {
			return false;
		}
	}

	// Use UTF-8 protocol with DB?
	// Will cause errors if DB tables were created with a different charset.
	// The updater will set 'set_charset' if the DB was converted correctly.
	if (isset($cfg_db['set_charset'])) {
		db_set_charset($cfg_db['set_charset']);
	} else {
		db_set_charset('latin1');
	}

	return $GLOBALS['dbconn'];
}

function db_select($conn, $dbname) {
	global $si;

	if ($si) {
		return mysqli_select_db($conn, $dbname);
	} else {
		return @mysql_select_db($dbname);
	}
}

function db_query($q) {
	global $si, $dbconn, $debug, $db_time_queries;

	db_push_history($q);
	$q_start = get_microtime();

	// Note: Queries do not trigger errors.  Do not mask with "@".
	if ($si) {
		$result = mysqli_query($dbconn, $q);
	} else {
		$result = mysql_query($q, $dbconn);
	}

	$q_time = get_microtime() - $q_start;
	$db_time_queries += $q_time;

	if ($result === false) {
		$marker = db_backtrace();
		db_push_error($marker.' '.db_last_error($dbconn));

		if ($debug) {
			echo ('<b>'.$marker.'</b> DB: '.db_last_error().' <b>(Q: '.htmlspecialchars(db_history()).')</b><br />');
		}
	}

	return $result;
}

function db_result($result, $offset = 0, $field = 0) {
	global $si, $dbconn, $debug;

	/*
		Note: avoid using field names with db_result(), as
		mysqli does not support this, and it's a bad thing
		to do.  I added this only for someone else's mod!
	*/

	$data = null;
	$array = null;

	if ($si) {
		if (@mysqli_data_seek($result, $offset)) {
			$array = mysqli_fetch_array($result, MYSQLI_NUM);
			$data = $array[0];
		}
	} else {
		if (empty($field)) {
			$data = @mysql_result($result, $offset);
		} else {
			$data = @mysql_result($result, $offset, $field);
		}
	}

	if ($data === false) {
		$marker = db_backtrace();
		db_push_error(db_last_error());

		if ($debug) {
			echo ('<b>'.$marker.'</b> DB: '.db_last_error().' <b>(Q: '.htmlspecialchars(db_history()).')</b><br />');
		} else {
			db_trigger();
		}
	}

	return $data;
}

function db_fetch_array($result, $type = null) {
	global $si, $dbconn;

	if (defined('MYSQLI_ASSOC') && empty($type)) {
		$type = MYSQLI_ASSOC;
	}

	/*
		Notes:

		- Unlike the native mysql functions, this function always
		returns associative arrays by default.

		- Due to the shifting internal pointer in mysql_fetch_array(),
		we cannot test for FALSE here.  Do not mask errors with "@".
	*/
	if ($si) {
		return mysqli_fetch_array($result, $type);
	} else {
		return mysql_fetch_array($result);
	}
}

function db_fetch_row($result) {
	global $si, $dbconn;

	/*
		Note: due to the shifting internal pointer in mysql_fetch_array(),
		we cannot test for FALSE here.  Do not mask errors with "@".
	*/
	if ($si) {
		return mysqli_fetch_row($result);
	} else {
		return mysql_fetch_row($result);
	}
}

function db_num_rows($result) {
	global $si, $dbconn;

	if ($si) {
		return mysqli_num_rows($result);
	} else {
		return mysql_num_rows($result);
	}
}

function db_affected_rows() {
	global $si, $dbconn;

	if ($si) {
		return mysqli_affected_rows($dbconn);
	} else {
		// WARNING: Make sure $dbconn is global, even though it is not passed.
		// PHP does some magic here, and passing $dbconn explicitly causes "trouble".
		return mysql_affected_rows();
	}
}

function db_insert_id() {
	global $si, $dbconn;

	if ($si) {
		return mysqli_insert_id($dbconn);
	} else {
		return mysql_insert_id();
	}
}

function db_free_result($result) {
	global $si, $dbconn;

	if ($si) {
		return @mysqli_free_result($result);
	} else {
		return @mysql_free_result($result);
	}
}

function db_escape($in) {
	global $si, $dbconn;

	if (is_array($in)) {
		if ($si) {
			return array_map('mysqli_real_escape_string', array_fill(0, count($in), $dbconn), $in);
		}
		return array_map('mysql_real_escape_string', $in, array_fill(0, count($in), $dbconn));
	}

	if ($si) {
		return mysqli_real_escape_string($dbconn, $in);
	}
	return mysql_real_escape_string($in, $dbconn);
}

function db_get_charset() {
	global $si, $dbconn;

	if ($si) {
		return mysqli_character_set_name($dbconn);
	} else {
		return mysql_client_encoding($dbconn);
	}
}

function db_set_charset($charset = 'utf8') {
	global $si, $dbconn;

	// MySQL is weird, as usual
	if ($charset == 'utf-8') {
		$charset = 'utf8';
	}
	if ($charset == 'iso-8859-1') {
		$charset = 'latin1';
	}

	// Requires MySQL 5.0.8
	if ($si) {
		return mysqli_set_charset($dbconn, $charset);
	} else {
		return @mysql_set_charset($charset);
	}
}

// Used in this script only
function db_last_error() {
	global $si, $dbconn;

	if ($dbconn) {
		if ($si) {
			return @mysqli_error($dbconn);
		} else {
			return mysql_error();
		}
	} else {
		return 'No connection';
	}
}

function db_error($offset = null) {
	global $db_error;

	if ($offset === null) {
		return end($db_error);
	}

	$offset = (int) $offset;
	if ($offset < 0 || $offset > count($db_error)) {
		trigger_error("db_error(): undefined offset ".$offset, E_USER_WARNING);
	} else {
		return $db_error[$offset];
	}
}

function db_num_errors() {
	global $db_error;
	return count($db_error);
}

function db_push_error($in) {
	global $db_error;

	if (!empty($in)) {
		$db_error[] = $in;
	}

	return;
}

function db_history($offset = null) {
	global $db_history;

	if ($offset === null) {
		return end($db_history);
	}

	$offset = (int) $offset;
	if ($offset < 0 || $offset > count($db_history)) {
		trigger_error("db_history(): undefined offset ".$offset, E_USER_WARNING);
	} else {
		return $db_history[$offset];
	}
}

function db_num_histories() {
	global $db_history;
	return count($db_history);
}

function db_push_history($in) {
	global $db_history;

	if (!empty($in)) {
		$db_history[] = $in;
	}

	return;
}

function db_backtrace() {
	// Gets backtrace to figure out line number for bad db_*() call.

	if (version_compare(PHP_VERSION, '5.4.0') >= 0) {
		$backtrace = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 2);
	} else {
		$backtrace = debug_backtrace();
	}
	$info = pathinfo($backtrace[1]['file']);

	return $info['basename'].'('.strval($backtrace[1]['line']).')::'.$backtrace[1]['function'];
}

function db_trigger() {
	global $debug;

	$marker = db_backtrace();

	trigger_error($marker, E_USER_WARNING);
	if ($debug) {
		echo '<b>DB: '.db_error().'</b><br />';
	}

	return;
}

function db_close($unused = 0) {
	// Don't global $dbconn here.  PHP may throw an undeclared variable error.
	global $si;

	$success = false;
	if (isset($GLOBALS['dbconn']) && $GLOBALS['dbconn']) {
		if ($si) {
			$success = @mysqli_close($GLOBALS['dbconn']);
		} else {
			$success = @mysql_close($GLOBALS['dbconn']);
		}
	} else {
		return true;
	}

	if ($success) {
		$GLOBALS['dbconn'] = false;
		return true;
	}

	return false;
}